//graphis.cpp
#include "graphics.h"


#pragma comment (lib, "dxguid.lib")
#pragma comment (lib, "d3d9.lib")
#pragma comment (lib, "dxerr9.lib")

#ifndef _DEBUG
#pragma comment (lib, "d3dx9d.lib")
#else
#pragma comment (lib, "d3dx9.lib")
#endif


void DebugError( char* szMsg )
{
    char pszError[256];

    wsprintf(pszError, "%s(%d): %s\n", __FILE__, __LINE__, szMsg);
    ::OutputDebugString(pszError);
} 



CGraphics::CGraphics()
{
	m_pD3D = NULL;
	m_pDevice = NULL;
}
//------------------------------------------------------------------------------------------
CGraphics::~CGraphics()
{
	Cleanup();
}
//------------------------------------------------------------------------------------------
void CGraphics::Cleanup()
{
	SAFE_RELEASE(m_pDevice);
	SAFE_RELEASE(m_pD3D);
}
//------------------------------------------------------------------------------------------
bool CGraphics::Create(HWND hWnd, int iWidth,int iHeight, bool bIsFullScreen)
{
	m_hWnd = hWnd;
	m_iWidth = iWidth;
	m_iHeight = iHeight;

	m_dmMode.Width = iWidth;
	m_dmMode.Height = iHeight;
	m_dmMode.RefreshRate = D3DPRESENT_RATE_DEFAULT;
	m_dmMode.Format = D3DFMT_A8R8G8B8;

	CreateDevice(bIsFullScreen);

	return true;
}
//------------------------------------------------------------------------------------------
HRESULT CGraphics::CreateDevice(bool bIsFullScreen)
{
	ZeroMemory (&m_d3dpp, sizeof(D3DPRESENT_PARAMETERS));
    if( NULL == ( m_pD3D = Direct3DCreate9( D3D_SDK_VERSION ) ) )
        return E_FAIL;

	//ؽ ó 
	D3DCAPS9 caps;
	D3DDEVTYPE deviceType = D3DDEVTYPE_HAL;
	m_pD3D->GetDeviceCaps(D3DADAPTER_DEFAULT, deviceType, &caps);
	int vertexProcessing = 0;
	if( caps.DevCaps & D3DDEVCAPS_HWTRANSFORMANDLIGHT )
		vertexProcessing = D3DCREATE_HARDWARE_VERTEXPROCESSING;
	else
		vertexProcessing = D3DCREATE_SOFTWARE_VERTEXPROCESSING;

	if(bIsFullScreen)
	{
		m_d3dpp.BackBufferWidth = m_dmMode.Width;
		m_d3dpp.BackBufferHeight = m_dmMode.Height;
		m_d3dpp.BackBufferCount = 1;
		m_d3dpp.FullScreen_RefreshRateInHz = m_dmMode.RefreshRate; 
	    m_d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_ONE;
	}
	else
	{
		if( FAILED( m_pD3D->GetAdapterDisplayMode (D3DADAPTER_DEFAULT, &m_dmMode)))
			return E_FAIL;
	}

	m_d3dpp.Windowed = (bIsFullScreen ? FALSE : TRUE);
    m_d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
    m_d3dpp.BackBufferFormat = m_dmMode.Format;
	m_d3dpp.hDeviceWindow              = m_hWnd;
	m_d3dpp.EnableAutoDepthStencil     = true; 
	m_d3dpp.AutoDepthStencilFormat     = D3DFMT_D24S8;

    if(FAILED(m_pD3D->CreateDevice (D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, m_hWnd, 
								   vertexProcessing, 
								   &m_d3dpp, &m_pDevice)))
	{
		return E_FAIL;
	}

	return S_OK;
}
//------------------------------------------------------------------------------------------
void CGraphics::SetupCamera()
{
    D3DXMATRIX   matProj;
	D3DXMATRIX	 matCameraView;
	D3DXVECTOR3  cameraTarget;
    D3DXVECTOR3  cameraPos;

	// Set where camera is viewing from
	cameraPos.x = 0.0f;
	cameraPos.y = 0.0f;
	cameraPos.z = -15.0f;
	// Set what camera is looking at
	cameraTarget.x = 0.0f;
	cameraTarget.y = 0.0f;
	cameraTarget.z = 0.0f;

	// Setup Camera View
	D3DXMatrixLookAtLH( &matCameraView, 
						&cameraPos, 
						&cameraTarget, 
						&D3DXVECTOR3( 0.0f, 1.0f, 0.0f ) );	

	m_pDevice->SetTransform( D3DTS_VIEW, &matCameraView );

	// Setup Projection 
	RECT rect;
	GetClientRect(m_hWnd, &rect);
	float width = (float)rect.right, height = (float)rect.bottom;
	D3DXMatrixPerspectiveFovLH( &matProj, D3DX_PI/4, width/height, 1.0f, 500.0f );
	m_pDevice->SetTransform( D3DTS_PROJECTION, &matProj );
}
//------------------------------------------------------------------------------------------
void CGraphics::BeginScene()
{
    m_pDevice->Clear (0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB (0, 0, 255), 1.0f, 0);
    m_pDevice->BeginScene ();
}
//------------------------------------------------------------------------------------------
HRESULT CGraphics::EndScene()
{	
	HRESULT hr;
    m_pDevice->EndScene ();
    hr = m_pDevice->Present (NULL, NULL, NULL, NULL);
	return hr;
}
//------------------------------------------------------------------------------------------
HRESULT CGraphics::RestoreDevice()
{
    HRESULT hr;

    // Test the cooperative level to see if it's okay to render
    if( FAILED( hr = m_pDevice->TestCooperativeLevel() ) )
    {
        // If the device was lost, do not render until we get it back
        if( hr == D3DERR_DEVICELOST)
            return hr;

        // Check if the device needs to be resized.
        if( hr == D3DERR_DEVICENOTRESET)
        {
            // If we are windowed, read the desktop mode and use the same format for the back buffer
            if( m_d3dpp.Windowed )
            {
				m_pD3D->GetAdapterDisplayMode (D3DADAPTER_DEFAULT, &m_dmMode);
                m_d3dpp.BackBufferFormat = m_dmMode.Format;
            }

            if( FAILED( hr = m_pDevice->Reset(&m_d3dpp) ) )
                return hr;
        }
        return hr;
    }

	return S_OK;
}
//-------------------------------------------------------------------------------------------------
IDirect3DDevice9 * CGraphics::GetDevice()  
{
	return m_pDevice;
}


