//testapp.cpp
#include <windows.h>
#include "./dx_base/graphics.h"
#include "testapp.h"
#include "./dx_util/xfile.h"
#include "./dx_util/dxeffect.h"
#include "./dx_util/MyRenderer.h"
#include "./dx_util/MyMesh.h"
#include "./dx_util/MyTexture.h"

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	CTestApp * pApp = new CTestApp;

	pApp->m_mainParam.hInstance = hInstance;
	pApp->m_mainParam.lpCmdLine = lpCmdLine;
	pApp->m_mainParam.nCmdShow = nCmdShow;
	
	pApp->Register(pApp, "XFile draw", 640, 480); 
	pApp->Run();
	pApp->Release();
	return 0;
}
//---------------------------------------------------------------------------------
CTestApp::CTestApp()
{
	m_pXFile = NULL;
	m_pEffect = NULL;
	m_cameraRef.wheel = 0.0f;
}
//---------------------------------------------------------------------------------
CTestApp::~CTestApp()
{

}
//---------------------------------------------------------------------------------
void CTestApp::InitDirectionalLight()
{
	IDirect3DDevice9 * pDevice = m_pGraphics->GetDevice();

	D3DLIGHT9 light;
	ZeroMemory( &light, sizeof(light) );
	light.Type = D3DLIGHT_DIRECTIONAL;

	D3DXVECTOR3 vecDir;
	vecDir = D3DXVECTOR3(1.0f, -1.0f, 0.0f);
	D3DXVec3Normalize((D3DXVECTOR3*)&light.Direction, &vecDir );

	// set directional light diffuse color
	light.Type  = D3DLIGHT_DIRECTIONAL;
	light.Diffuse = D3DXCOLOR( 1.f, 1.f, 1.f, 1.f );
	light.Ambient = D3DXCOLOR( 0.f, 0.f, 0.f, 1.f );

	pDevice->SetLight( 0, &light );
	pDevice->LightEnable( 0, TRUE );
	pDevice->SetRenderState( D3DRS_LIGHTING, TRUE );
	pDevice->SetRenderState( D3DRS_ZENABLE,  TRUE );
	//Ambient Light 
	pDevice->SetRenderState(D3DRS_AMBIENT, 0x00202020);
}
//---------------------------------------------------------------------------------
bool CTestApp::PostGraphics()
{
	InitDirectionalLight();

	IDirect3DDevice9 * pDevice = m_pGraphics->GetDevice();
	
	m_pXFile = new CXFile;
	m_pXFile->Load(pDevice, "airplane 2.x");

	m_pRenderer = new MyRenderer;
	m_pMesh = new MyMesh;
	m_pTexture = new MyTexture("kim.jpg");

	MyVector3* pVertex = new MyVector3[6];
	pVertex[0] = MyVector3(-1.0f, -1.0f, 5.0f);
	pVertex[1] = MyVector3(-1.0f, 1.0f, 5.0f);
	pVertex[2] = MyVector3(1.0f, -1.0f, 5.0f);
	pVertex[3] = MyVector3(-1.0f, 1.0f, 5.0f);
	pVertex[4] = MyVector3(1.0f, 1.0f, 5.0f);
	pVertex[5] = MyVector3(1.0f, -1.0f, 5.0f);

	MyVector2* pUV = new MyVector2[6];
	pUV[0] = MyVector2( 0.0f, 1.0f);
	pUV[1] = MyVector2( 0.0f, 0.0f);
	pUV[2] = MyVector2( 1.0f, 1.0f);
	pUV[3] = MyVector2( 0.0f, 0.0f);
	pUV[4] = MyVector2( 1.0f, 0.0f);
	pUV[5] = MyVector2( 1.0f, 1.0f);

	m_pMesh->SetNumber( 6, 2);
	m_pMesh->SetVertexData( pVertex);
	m_pMesh->SetTextureUV( pUV); 

	return true;
}
//---------------------------------------------------------------------------------
void CTestApp::Cleanup()
{
	SAFE_DELETE(m_pEffect);
	SAFE_DELETE(m_pXFile);
	SAFE_DELETE(m_pTexture);
	SAFE_DELETE(m_pMesh);
	SAFE_DELETE(m_pRenderer);
	CApplication::Cleanup();
}
//---------------------------------------------------------------------------------
void CTestApp::Update()
{
}
//---------------------------------------------------------------------------------
void CTestApp::Render()
{
	D3DXMATRIX   matTranslation, matRot, matWorld;
	D3DXMatrixTranslation( &matTranslation, 0.0f, 0.0f, m_cameraRef.wheel * 2.0f);
	D3DXMatrixRotationYawPitchRoll( &matRot, D3DXToRadian(m_cameraRef.fRotateX), D3DXToRadian(m_cameraRef.fRotateY), 0.0f );
	IDirect3DDevice9 * pDevice = m_pGraphics->GetDevice();

	matWorld = matRot * matTranslation;
	pDevice->SetTransform( D3DTS_WORLD, &matWorld);
	m_pXFile->Draw();

	//m_pMesh 
	pDevice->SetRenderState( D3DRS_LIGHTING, FALSE );

	D3DXMATRIX world;
    D3DXMatrixTranslation( &world, 1.5f, -1.0f, -10.0f );
	m_pTexture->SetTexture( pDevice);
    pDevice->SetTransform( D3DTS_WORLD, &world );
	m_pRenderer->Render(pDevice, *m_pMesh);
}
//---------------------------------------------------------------------------------
bool CTestApp::ProcessKey(HWND hWnd, WPARAM wKey)
{
	return CApplication::ProcessKey(hWnd, wKey);
}


